@@ -1,7 +1,7 @@
 #! /usr/bin/env perl
 
 #
-#   Copyright (C) Dr. Heinz-Josef Claes (2003-2012)
+#   Copyright (C) Dr. Heinz-Josef Claes (2003-2022)
 #                 hjclaes@web.de
 #
 #   This program is free software: you can redistribute it and/or modify
@@ -19,9 +19,6 @@
 #
 
 
-my $VERSION = '$Id: storeBackupDel.pl 362 2012-01-28 22:11:13Z hjc $ ';
-push @VERSION, $VERSION;
-my ($VERSIONpName, $VERSIONsvnID) = $VERSION =~ /Id:\s+(\S+)\s+(\d+)/;
 $main::STOREBACKUPVERSION = undef;
 
 
@@ -90,7 +87,8 @@ sub splitFileDir
     return ($dir, $file);
 }
 my ($req, $prog) = &libPath($0);
-(@INC) = ($req, @INC);
+unshift @INC, "$req";
+
 
 require 'storeBackupLib.pl';
 require 'checkParam2.pl';
@@ -101,7 +99,9 @@ require 'dateTools.pl';
 require 'fileDir.pl';
 require 'humanRead.pl';
 
-my $lockFile = '/tmp/storeBackup.lock';   # default value
+my $lockFile = ($ENV{'USER'} eq 'root') ?     # default value
+		'/var/lock/storeBackup.lock' :
+		$ENV{'HOME'} . '/storeBackup.lock';
 my $keepAll = '30d';
 my $keepDuplicate = '7d';
 my $checkSumFile = '.md5CheckSums';
@@ -155,6 +155,11 @@ storeBackupDel.pl - this program deletes backups created by storeBackup
 
     lock file, if exists, new instances will finish if
     an old is already running, default is $lockFile
+    this type of lock files does not work across multiple servers
+    and is not designed to separate storeBackup.pl and
+    storeBackupUpdateBackup.pl or any other storeBackup
+    process in a separate PID space
+    Default lock file for root is in /var/lock, for others in \$HOME
 
 =item B<--doNotDelete>
 
@@ -254,7 +259,7 @@ storeBackupDel.pl - this program deletes backups created by storeBackup
     Alternative deletion scheme. If you use this option, all other
     keep options are ignored. Preserves backups depending
     on their *relative* age. Example:
-    -R '1d 7d 2m 3m'
+    -R '1d 7d 61d 92d'
         will (try to) ensure that there is always
 	- One backup between 1 day and 7 days old
 	- One backup between 5 days and 2 months old
@@ -287,7 +292,7 @@ storeBackupDel.pl - this program deletes backups created by storeBackup
 =item B<--saveLogs>
 
     save log files with date and time instead of deleting the
-    old (with [-noOldFiles])
+    old (with [-noOfOldFiles])
 
 =item B<--compressWith>
 
@@ -298,16 +303,14 @@ storeBackupDel.pl - this program deletes backups created by storeBackup
 
 =head1 COPYRIGHT
 
-Copyright (c) 2003-2008 by Heinz-Josef Claes (see README).
+Copyright (c) 2003-2022 by Heinz-Josef Claes (see README).
 Published under the GNU General Public License v3 or any later version
 
 =cut
 
-my $Help = join('', grep(!/^\s*$/, `pod2text $0`));
-$Help = "cannot find pod2text, see documentation for details\n"
-    unless $Help;
+my $Help = &::getPod2Text($0);
 
-&printVersions(\@ARGV, '-V');
+&printVersion(\@ARGV, '-V', '--version');
 
 my $startDate = dateTools->new();
 
@@ -396,6 +399,7 @@ my $CheckPar =
 					    '-pattern' => '\A\d+\Z'),
                                 Option->new('-name' => 'keepRelative',
 					    '-cl_option' => '--keepRelative',
+					    '-cl_alias' => '-R',
 					    '-cf_key' => 'keepRelative',
 					    '-quoteEval' => 'yes',
 					    '-param' => 'yes'),
@@ -446,7 +450,12 @@ my $CheckPar =
 					    '-cl_option' => '--today',
 					    '-cf_key' => 'today',
 					    '-hidden' => 'yes',
-					    '-param' => 'yes')
+					    '-param' => 'yes'),
+# used by storeBackupMount.pl
+				Option->new('-name' => 'writeToNamedPipe',
+					    '-cl_option' => '--writeToNamedPipe',
+					    '-param' => 'yes',
+					    '-hidden' => 'yes')
 				]
 		    );
 
@@ -487,10 +496,11 @@ my $maxFilelen = $CheckPar->getOptWithPar('maxFilelen');
 my $noOfOldFiles = $CheckPar->getOptWithPar('noOfOldFiles');
 my $saveLogs = $CheckPar->getOptWithoutPar('saveLogs') ? 'yes' : 'no';
 my $compressWith = $CheckPar->getOptWithPar('compressWith');
-
+# hidden options
 my $printAll = $CheckPar->getOptWithoutPar('printAll');
 $print = 1 if $printAll;
 my $todayOpt = $CheckPar->getOptWithPar('todayOpt');  # format like
+my $writeToNamedPipe = $CheckPar->getOptWithPar('writeToNamedPipe');
                                                       # backup dir name
 
 if ($print)
@@ -499,26 +509,48 @@ if ($print)
     exit 0;
 }
 
-my $prLog;
-my (@kind) = ('I:INFO', 'W:WARNING', 'E:ERROR', 'S:STATISTIC',
-	      'D:DEBUG', 'V:VERSION');
+my $prLog1;
+my (@kind) = ('A:BEGIN', 'Z:END','I:INFO', 'W:WARNING', 'E:ERROR',
+	      'S:STATISTIC', 'D:DEBUG', 'V:VERSION');
 
 if ($logFile)
 {
-    $prLog = printLog->new('-kind' => \@kind,
-			   '-file' => $logFile,
-			  '-withTime' => $withTime,
-			  '-maxFilelen' => $maxFilelen,
-			  '-noOfOldFiles' => $noOfOldFiles);
+    $prLog1 = printLog->new('-kind' => \@kind,
+			    '-file' => $logFile,
+			    '-withTime' => $withTime,
+			    '-maxFilelen' => $maxFilelen,
+			    '-noOfOldFiles' => $noOfOldFiles);
 }
 else
 {
-    $prLog = printLog->new('-kind' => \@kind);
+    $prLog1 = printLog->new('-kind' => \@kind);
 }
 
+my $prLog = printLogMultiple->new('-prLogs' => [$prLog1]);
+
+if ($plusLogStdout)
+{
+    my $p = printLog->new('-kind' => \@kind,,
+			  '-filedescriptor', *STDOUT);
+    $prLog->add('-prLogs' => [$p]);
+}
+if ($writeToNamedPipe)
+{
+    my $np = printLog->new('-kind' => \@kind,
+			   '-file' => $writeToNamedPipe,
+			   '-maxFilelen' => 0);
+    $prLog->add('-prLogs' => [$np]);
+}
+
+(@main::cleanup) = ($prLog, undef);
+$SIG{INT} = \&cleanup;
+$SIG{TERM} = \&cleanup;
+
+$prLog->print('-kind' => 'A',
+	      '-str' =>
+	      ["starting deletion in <$backupDir>, series <$series>"]);
 $prLog->print('-kind' => 'V',
-	      '-str' => ["$VERSIONpName, $main::STOREBACKUPVERSION, " .
-			 "build $VERSIONsvnID"]);
+	      '-str' => ["storeBackupDel.pl, $main::STOREBACKUPVERSION"]);
 
 $prLog->print('-kind' => 'E',
 	      '-str' => ["backupDir directory <$backupDir> does not exist\n$Help"],
@@ -542,44 +574,7 @@ my $allLinks = lateLinks->new('-dirs' => [$targetDir],
 #
 # lock file berprfen
 #
-if ($lockFile)
-{
-    if (-f $lockFile)
-    {
-	open(FILE, "< $lockFile") or
-	    $prLog->print('-kind' => 'E',
-			  '-str' => ["cannot read lock file <$lockFile>"],
-			  '-exit' => 1);
-	my $pid = <FILE>;
-	chop $pid;
-	close(FILE);
-	$prLog->print('-kind' => 'E',
-		      '-str' => ["strange format in lock file <$lockFile>, " .
-				 "line is <$pid>"],
-		      '-exit' => 1)
-	    unless ($pid =~ /\A\d+\Z/o);
-	if (kill(0, $pid) == 1)   # alte Instanz luft noch
-	{
-	    $prLog->print('-kind' => 'E',
-			  '-str' => ["cannot start, old instance with pid " .
-				     "<$pid> is already running"],
-			  '-exit' => 1);
-	}
-	else
-	{
-	    $prLog->print('-kind' => 'I',
-			  '-str' => ["removing old lock file of process <$pid>"]
-			  );
-	}
-    }
-
-    open(FILE, "> $lockFile") or
-	$prLog->print('-kind' => 'E',
-		      '-str' => ["cannot create lock file <$lockFile>"],
-		      '-exit' => 1);
-    print FILE "$$\n";
-    close(FILE);
-}
+::checkLockFile($lockFile, $prLog);
 
 my $statDelOldBackupDirs =
     statisticDeleteOldBackupDirs->new('-prLog' => $prLog);
@@ -677,4 +672,20 @@ $prLog->print('-kind' => 'I',
 	      '-str' => ["removing lock file <$lockFile>"]);
 unlink $lockFile;
 
+$prLog->print('-kind' => 'Z',
+	      '-str' =>
+	      ["finished deletion in <$backupDir>, series <$series>"]);
+
+
 exit 0;
+
+
+##################################################
+# package printLogMultiple needs this function
+sub cleanup
+{
+    my $signame = shift;
+    my $exit = (shift() || 1);
+
+    exit $exit;
+}
