@@ -1,7 +1,7 @@
 #! /usr/bin/env perl
 
 #
-#   Copyright (C) Dr. Heinz-Josef Claes (2002-2012)
+#   Copyright (C) Dr. Heinz-Josef Claes (2002-2022)
 #                 hjclaes@web.de
 #   
 #   This program is free software: you can redistribute it and/or modify
@@ -19,9 +19,6 @@
 #
 
 
-my $VERSION = '$Id: storeBackupVersions.pl 362 2012-01-28 22:11:13Z hjc $ ';
-push @VERSION, $VERSION;
-my ($VERSIONpName, $VERSIONsvnID) = $VERSION =~ /Id:\s+(\S+)\s+(\d+)/;
 $main::STOREBACKUPVERSION = undef;
 
 use Fcntl qw(O_RDWR O_CREAT);
@@ -91,7 +88,7 @@ sub splitFileDir
     return ($dir, $file);
 }
 my ($req, $prog) = &libPath($0);
-push @INC, "$req";
+unshift @INC, "$req";
 
 require 'checkParam2.pl';
 require 'checkObjPar.pl';
@@ -174,16 +171,14 @@ non-blocked in different backups. In such cases, use option B<--locateSame>.
 
 =head1 COPYRIGHT
 
-Copyright (c) 2002-2008 by Heinz-Josef Claes (see README)
+Copyright (c) 2002-2022 by Heinz-Josef Claes (see README)
 Published under the GNU General Public License v3 or any later version
 
 =cut
 
-my $Help = join('', grep(!/^\s*$/, `pod2text $0`));
-$Help = "cannot find pod2text, see documentation for details\n"
-    unless $Help;
+my $Help = &::getPod2Text($0);
 
-&printVersions(\@ARGV, '-V');
+&printVersion(\@ARGV, '-V', '--version');
 
 my $CheckPar =
     CheckParam->new('-allowLists' => 'no',
@@ -271,8 +266,7 @@ my $prLog = printLog->new('-kind' => ['I:INFO', 'W:WARNING', 'E:ERROR',
 				      'S:STATISTIC', 'D:DEBUG', 'V:VERSION']);
 
 $prLog->print('-kind' => 'V',
-	      '-str' => ["$VERSIONpName, $main::STOREBACKUPVERSION, " .
-			 "build $VERSIONsvnID"])
+	      '-str' => ["storeBackupVersions.pl, $main::STOREBACKUPVERSION"])
     if $verbose;
 
 #
@@ -327,7 +321,7 @@ $checkSumFileRoot .= ".bz2" if (-f "$backupRoot/$checkSumFile.bz2");
 $prLog->print('-kind' => 'E',
 	      '-str' => ["no info file <$checkSumFileRoot> in <$backupRoot>"],
 	      '-exit' => 1)
-    unless(-f "$backupRoot/$checkSumFileRoot");
+    unless (-f "$backupRoot/$checkSumFileRoot");
 
 # jetzt $restoreTree relativ zu $backupRoot machen
 my $fileWithRelPath = substr($file, length($backupRoot) + 1);
