@@ -1,7 +1,7 @@
 # -*- perl -*-
 
 #
-#   Copyright (C) Heinz-Josef Claes (2000-2012)
+#   Copyright (C) Heinz-Josef Claes (2000-2022)
 #                 hjclaes@web.de
 #   
 #   This program is free software: you can redistribute it and/or modify
@@ -20,8 +20,6 @@
 
 
 
-push @VERSION, '$Id: dateTools.pl 362 2012-01-28 22:11:13Z hjc $ ';
-
 use strict;
 
 require 'checkObjPar.pl';
@@ -279,6 +277,29 @@ sub valToStr
 }
 
 
+# multiplies something like '4d2h31s' with a factor
+# and returns eg. '8d4h1m2s' if factor was 2
+sub multiplyWithStr
+{
+    # set default value
+    my (%params) = ('-str'  => undef,
+		    '-factor' => undef
+		    );
+
+    &::checkObjectParams(\%params, \@_, 'dateTools::valToStr', []);
+
+    my $str = $params{'-str'};
+    my $factor = $params{'-factor'};
+
+    return undef
+	if not defined($str) or not defined($factor);
+
+    my $sec = &dateTools::strToSec('-str' => $str);
+    $sec = int($sec * $factor + .5);
+    return &dateTools::valToStr('-sec' => $sec);
+}
+
+
 # Returnwerte:
 #   -1 wenn Datum in Parameter vor Datum im Objekt
 #   +1 wenn Datum in Parameter nach Datum im Objekt
@@ -1128,12 +1149,12 @@ sub sub
 # x - d1 - s1 = x - s1 - (1day - s2)
 #             = x - d1 - 1day + s2
 
-    my $s1 = $subSec + 60 * ($subMin + 24 * $subHour);
+    my $s1 = $subSec + 60 * ($subMin + 60 * $subHour);
     my $s2 = 86400 - $s1;           # Restzeit in Sek bis vollen Tag
     $self->add('-sec' => $s2);
     $subDay++;
 
-# jetzt noch $subDay abziehen ###########################?????????
+# jetzt noch $subDay abziehen
     my $restDaysInYear = $self->getDaysInYear() - $self->getAktDaysInYear();
     $subDay += $restDaysInYear;
 
