@@ -1,7 +1,7 @@
 # -*- perl -*-
 
 #
-#   Copyright (C) Heinz-Josef Claes (2001-2012)
+#   Copyright (C) Heinz-Josef Claes (2001-2022)
 #                 hjclaes@web.de
 #
 #   This program is free software: you can redistribute it and/or modify
@@ -20,10 +20,11 @@
 
 
 
-push @VERSION, '$Id: fileDir.pl 364 2012-02-12 14:14:44Z hjc $ ';
-
+use Digest::MD5 qw(md5_hex);
 use Fcntl qw(O_RDWR O_CREAT);
+use Fcntl ':mode';
 use POSIX;
+use Cwd 'abs_path';
 
 require 'prLog.pl';
 require 'forkProc.pl';
@@ -40,21 +41,32 @@ sub copyFile
 {
     my $source = shift;
     my $target = shift;
+    my $prLog = shift;
 
     local *SOURCE;
     local *TARGET;
     unless (sysopen(SOURCE, $source, O_RDONLY))
     {
+	$prLog->print('-kind' => 'E',
+		      '-str' =>
+	  ["cannot open <$source> for reading and copying to <$target>"]);
 	return 0;
     }
     unless (sysopen(TARGET, $target, O_CREAT | O_WRONLY))
     {
+	$prLog->print('-kind' => 'E',
+		      '-str' => ["cannot create <$target> for writing"]);
 	return 0;
     }
     my $buffer;
-    while (sysread(SOURCE, $buffer, 10*1024**2))
+    while (sysread(SOURCE, $buffer, 16*1024**2))
     {
-	syswrite(TARGET, $buffer);
+	unless (syswrite(TARGET, $buffer))
+	{
+	    $prLog->print('-kind' => 'E',
+			  '-str' => [$!]);
+	    return 0;
+	}
     }
 
     close(TARGET);
@@ -63,6 +75,119 @@ sub copyFile
     return 1;
 }
 
+############################################################
+sub copySymLink
+{
+    my $source = shift;    # symlink source
+    my $target = shift;    # symlink target
+
+    my $l = readlink $source;
+    unless ($l)
+    {
+	return 0;
+    }
+    unless (symlink $l, $target)
+    {
+	return 0;
+    }
+    else
+    {
+	return 1;
+    }
+}
+
+
+############################################################
+# use cp_a to copy a directory
+# $ignoreError = 1 -> ignore error
+# $ignoreError = 0 -> stop at error
+sub copyDir
+{
+    my ($from, $to, $tmp, $prLog, $ignoreError) = (@_);
+
+    my $cp = forkProc->new('-exec' => 'cp',
+			   '-param' =>
+			   ['-r', "$from", "$to"],
+			   '-outRandom' => $tmp,
+			   '-prLog' => $prLog);
+    $cp->wait();
+    my $out = $cp->getSTDOUT();
+    if (@$out)
+    {
+	$prLog->print('-kind' => 'W',
+		      '-str' =>
+		      ["copying of <$from> to <$to> reported:",
+		       @$out]);
+    }
+    $out = $cp->getSTDERR();
+    if (@$out)
+    {
+	$prLog->print('-kind' => 'E',
+		      '-str' =>
+		      ["copying (with 'cp -r') of <$from> to <$to> reported:",
+		       @$out]);
+	exit 1 unless $ignoreError;
+    }
+}
+
+
+############################################################
+# use mv to move a directory
+sub mvComm
+{
+    my ($from, $to, $tmp, $prLog) = (@_);
+
+    my $cp = forkProc->new('-exec' => 'mv',
+			   '-param' =>
+			   ["$from", "$to"],
+			   '-outRandom' => $tmp,
+			   '-prLog' => $prLog);
+    $cp->wait();
+    my $out = $cp->getSTDOUT();
+    if (@$out)
+    {
+	$prLog->print('-kind' => 'W',
+		      '-str' =>
+		      ["moving of <$from> to <$to> reported:",
+		       @$out]);
+    }
+    $out = $cp->getSTDERR();
+    if (@$out)
+    {
+	$prLog->print('-kind' => 'E',
+		      '-str' =>
+		      ["moving of <$from> to <$to> reported:",
+		       @$out],
+		      '-exit' => 1);
+    }
+}
+
+
+
+############################################################
+# &::readDirStbu($dir, __FILE__, __LINE__, $prLog);
+sub readDirStbu
+{
+    my ($dir, $file, $line, $pattern, $prLog) = (@_);
+
+    local *DIR;
+    opendir(DIR, "$dir") or
+	$prLog->print('-kind' => 'E',
+		      '-str' =>
+		      ["cannot opendir <$dir>, exiting"],
+		      '-add' => [$file, $line],
+		      '-exit' => 1);
+    my (@dirs, $entry);
+    while ($entry = readdir DIR)
+    {
+	next unless $entry =~ /$pattern/;
+	push @dirs, $entry;
+    }
+    closedir(DIR);
+
+    return (sort @dirs);
+}
+
 
 # checks if a file is a symlink and deletes it if wanted
 # return values (if not exiting):
@@ -110,91 +235,38 @@ sub splitFileDir
     my $name = shift;
 
     return (undef, undef) unless $name;
-    return ('.', $name) unless ($name =~/\//);    # nur einfacher Dateiname
+    return ('.', $name) unless ($name =~/\//);    # simple file name only
 
     my ($dir, $file) = $name =~ /^(.*)\/(.*)$/s;
-    $dir = '/' if ($dir eq '');                   # gilt, falls z.B. /filename
+    $dir = '/' if ($dir eq '');                   # if eg. /filename
     return ($dir, $file);
 }
 
 
 ############################################################
-# Parameter kann directory oder Datei sein
+# Parameter may be directory or file
 %main::absolutePathCache = ();
 @main::absolutePathCache = ();
 sub absolutePath
 {
     my $dir = shift;
-
-    return undef unless $dir;
-
-    # Erst im Hash nachsehen, ob das schon ermittelt wurde
-    return $main::absolutePathCache{$dir}
-        if exists $main::absolutePathCache{$dir};
     my $dirSave = $dir;
 
-    # Falls Datei selbst ein symlink ist, solange folgen, bis aufgelst
-    my $file = '';
+    return undef unless $dir and -e $dir;
 
-
-    $dir = $ENV{'PWD'} . "/$dir" unless ($dir =~ /^\//); # relativer Pfad
-
-    if (-e $dir)
-    {
-	$file = $dir;
-	while (-l $file)
-	{
-	    my $link = readlink($file);
-
-	    if (substr($link, 0, 1) ne "/")
-	    {
-		$file =~ s/[^\/]+$/$link/;
-	    }
-	    else
-	    {
-		$file = $link;
-	    }
-	}
-
-	($dir, $file) = &splitFileDir($file);
-	$file = "/$file";
-    }
-
-    my $ret = (-e "$dir$file") ? "$dir$file" : undef;
-    $ret =~ s/\/+/\//g;                # // -> /
-    while ($ret =~ s/\/\.\//\//g) {}   # /./ -> /
-    $ret =~ s/\A(.+?)\/+\Z/$1/;        # remove trailing /
-    $ret =~ s/\A\.\///;                # ./xxx -> xxx
-    $ret =~ s/\/\.\Z//;                # xxx/. -> xxx
-
-    # eliminate /../
-    my (@ret) = split(/\//, $ret);
-    if (@ret)
-    {
-	my (@rev) = reverse(@ret);
-	(@ret) = ();
-	my ($i, $j);
-	for ($i = $j = 0 ; $i < @rev ; $i++, $j++)
-	{
-	    if ($rev[$i] eq '..')
-	    {
-		$j++;
-	    }
-	    else
-	    {
-		push @ret, $rev[$j] if ($j < @rev);
-	    }
-	}
-	$ret = join('/', reverse @ret);
-    }
+    # First look up in the hash table if that's already calculated
+    return $main::absolutePathCache{$dir}
+        if exists $main::absolutePathCache{$dir};
 
     if (@main::absolutePathCache > 100)
     {
 	my $del = shift @main::absolutePathCache;   # delete oldest
 	delete $main::absolutePathCache{$del};
     }
+    my $ret = abs_path($dir);
     push @main::absolutePathCache, $ret;
-    return $main::absolutePathCache{$dirSave} = $ret;
+    $main::absolutePathCache{$dirSave} = $ret;
+    return $ret;
 }
 
 
@@ -238,7 +310,7 @@ sub relPath
 ############################################################
 sub uniqFileName
 {
-    my $prefix = shift;                 # z.B. '/tmp/test-'
+    my $prefix = shift;                 # eg. '/tmp/test-'
 
     my $suffix;
     do
@@ -252,7 +324,7 @@ sub uniqFileName
 
 
 ############################################################
-# testet, ob subDir ein Unterverzeichnis von dir ist
+# tests if subDir is a sub directory of dir
 sub isSubDir
 {
     my $dir = shift;
@@ -305,11 +377,15 @@ sub makeFilePath
 
 ############################################################
 # like `mkdir -p`, all permissions set to 0700
+# success: returns 1
+# no success: returns 0
 sub makeDirPath
 {
     my $path = shift;
     my $prLog = shift;
 
+    return unless $path;
+
     my @p;
     return if -e $path;
 
@@ -322,8 +398,9 @@ sub makeDirPath
 	return if -e $path;
 	$prLog->print('-kind' => 'E',
 		      '-str' => ["cannot create directory <$path>"]);
+	return 0;
     }
-    return;
+    return 1;
 }
 
 
@@ -373,13 +450,19 @@ sub checkLockFile
     local *FILE;
     if (-f $lockFile)
     {
-	open(FILE, "< $lockFile") or
+	open(FILE, '<', $lockFile) or
 	    $prLog->print('-kind' => 'E',
 			  '-str' => ["cannot read lock file <$lockFile>"],
 			  '-exit' => 1);
 	my $pid = <FILE>;
 	chop $pid;
 	close(FILE);
+	#print "------------------------------$pid  $$\n";
+	if ($pid == $$)
+	{
+	#    print "------------------+++++++++++++++++++self llock\n";
+	    return;
+	}
 	$prLog->print('-kind' => 'E',
 		      '-str' => ["strange format in lock file <$lockFile>, " .
 				 "line is <$pid>"],
@@ -398,13 +481,19 @@ sub checkLockFile
 			  '-str' => ["removing old lock file of process <$pid>"]
 			  );
 	}
+	unless (unlink $lockFile)
+	{
+	    $prLog->print('-kind' => 'E',
+			  '-str' => ["cannot remove lock file <$lockFile"],
+			  '-exit' => 1);
+	}
     }
 
     $prLog->print('-kind' => 'I',
 		  '-str' => ["creating lock file <$lockFile>"]);
 
     &::checkDelSymLink($lockFile, $prLog, 0x01);
-    open(FILE, "> $lockFile") or
+    open(FILE, '>', $lockFile) or
 	$prLog->print('-kind' => 'E',
 		      '-str' => ["cannot create lock file <$lockFile>"],
 		      '-exit' => 1);
@@ -422,6 +511,9 @@ sub calcMD5
     local *FROM;
     unless (sysopen(FROM, $filename, O_RDONLY))
     {
+	$prLog->print('-kind' => 'E',
+		      '-str' => ["cannot open <$filename> in calcMD5"])
+	    if $prLog;
 	return undef;
     }
     my $md5 = Digest::MD5->new();
@@ -435,6 +527,266 @@ sub calcMD5
 }
 
 
+############################################################
+sub readSymLinkCalcMd5
+{
+    my $filename = shift;
+    my $prLog = shift;
+
+    my $l = readlink $filename;
+    unless ($l)
+    {
+	$prLog->print('-kind' => 'E',
+		      '-str' => ["cannot read symlink value <$filename>"]);
+	return undef;
+    }
+    my $md5 = Digest::MD5->new();
+    $md5->add($l);
+    return $md5->hexdigest();
+}
+
+############################################################
+sub createArchiveFromFile
+{
+    my $filename = shift;   # relative path
+    my $sourceDir = shift;
+    my $backupDir = shift;
+    my $archiver = shift;
+    my $prLog = shift;
+    my $tmpdir = shift;
+
+    my $prog = $main::fileTypeArchiver{$archiver}{'prog'};
+    my $opts = $main::fileTypeArchiver{$archiver}{'createOpts'}; # \@list
+    my ($pathToSourceFile, $f) = &::splitFileDir("$sourceDir/$filename");
+
+    if ($archiver eq 'tar')
+    {
+	push @$opts, '-', $f;
+	my $tar = forkProc->new('-exec' => $prog,
+				'-param' => $opts,
+				'-workingDir' => $pathToSourceFile,
+				'-stdout' => "$backupDir/$filename",
+				'-stderr' => "$tmpdir/stbuPipeArchive-",
+				'-delStdout' => 'no',
+				'-prLog' => $prLog);
+	$tar->wait();
+	my $out = $tar->getSTDOUT();
+	$prLog->print('-kind' => 'W',
+		      '-str' => ["STDOUT of <$prog @$opts>:", @$out])
+	    if (@$out > 0);
+	$out = $tar->getSTDERR();
+	$prLog->print('-kind' => 'E',
+		      '-str' => ["STDERR of <$prog @$opts>:",
+				 @$out])
+	    if (@$out > 0);
+    }
+    elsif ($archiver eq 'cpio')
+    {
+	my $cpio = pipeToFork->new('-exec' => $prog,
+				   '-param' => $opts,
+				   '-workingDir' => $pathToSourceFile,
+				   '-stdout' => "$backupDir/$filename",
+				   '-stderr' => "$tmpdir/stbuPipeArchive-",
+				   '-delStdout' => 'no',
+				   '-prLog' => $prLog);
+	$cpio->print("$f\n");
+	$cpio->wait();
+	my $out = $cpio->getSTDERR();
+	$prLog->print('-kind' => 'E',
+		      '-str' => ["STDERR of <$prog @$opts>:", @$out])
+	    if (@$out > 0);
+    }
+    else
+    {
+	$prLog->print('-kind' => 'E',
+		      '-str' => ["archiver <$archiver> unknown, exiting"],
+		      '-exit' => 1);
+    }
+}
+
+
+############################################################
+sub extractFileFromArchive
+{
+    my $filename = shift;   # relative path
+    my $restoreDir = shift;
+    my $backupDir = shift;
+    my $archiver = shift;
+    my $prLog = shift;
+    my $tmpdir = shift;
+
+#print "+++extractFileFromArchive:+$filename+$restoreDir+$backupDir+$archiver+\n";
+    my $prog = $main::fileTypeArchiver{$archiver}{'prog'};
+    my $opts = $main::fileTypeArchiver{$archiver}{'extractOpts'}; # \@list
+    my ($pathToRestoreDir, $f) = &::splitFileDir("$restoreDir/$filename");
+
+    if ($archiver eq 'tar')
+    {
+	push @$opts, "$backupDir/$filename";
+#print "+++$prog @$opts+++\n";
+	my $tar = forkProc->new('-exec' => $prog,
+				'-param' => $opts,
+				'-workingDir' => $pathToRestoreDir,
+				'-outRandom' => "$tmpdir/stbuPipeArchive-",
+				'-prLog' => $prLog);
+	$tar->wait();
+	my $out = $tar->getSTDOUT();
+	$prLog->print('-kind' => 'W',
+		      '-str' => ["STDOUT of <$prog @$opts>:", @$out])
+	    if (@$out > 0);
+	$out = $tar->getSTDERR();
+	$prLog->print('-kind' => 'E',
+		      '-str' => ["STDERR of <$prog @$opts>:", @$out])
+	    if (@$out > 0);
+    }
+    elsif ($archiver eq 'cpio')
+    {
+#print "+++$prog @$opts+++\n";
+	my $cpio = forkProc->new('-exec' => $prog,
+				 '-param' => $opts,
+				 '-workingDir' => $pathToRestoreDir,
+				 '-stdin' => "$backupDir/$filename",
+				 '-outRandom' => "$tmpdir/stbuPipeArchive-",
+				 '-prLog' => $prLog);
+	$cpio->wait();
+	my $out = $cpio->getSTDOUT();
+	$prLog->print('-kind' => 'W',
+		      '-str' => ["STDOUT of <$prog @$opts>:", @$out])
+	    if (@$out > 0);
+	$out = $cpio->getSTDERR();
+	$prLog->print('-kind' => 'E',
+		      '-str' => ["STDERR of <$prog @$opts>:", @$out])
+	    if (@$out > 0);
+    }
+    else
+    {
+	$prLog->print('-kind' => 'E',
+		      '-str' => ["archiver <$archiver> unknown, exiting"],
+		      '-exit' => 1);
+    }
+#print "+++end of extractFileFromArchive:+$filename+\n";
+}
+
+
+############################################################
+# helper functions for &main::MARK_DIR and &main::MARK_DIR_REC
+# problem is to store 2 dimensional hash with dbm files (saveRAM)
+# simulates $a{$b}{$c} = 1 or $a{$b}{$d} = 1
+# with $a{$b} = 'SEP$cSEP$aSEP' where SEPerator is \000
+sub addToStr
+{
+    my $hash = shift;
+    my $k1 = shift;
+    my $k2 = shift;
+
+    if (exists $$hash{$k1})
+    {
+	if (index($$hash{$k1}, "\000$k2\000") < 0)  # not found
+	{
+	    $$hash{$k1} = $$hash{$k1} . "$k2\000";
+	}
+    }
+    else
+    {
+	$$hash{$k1} = "\000$k2\000";
+    }
+}
+
+########################################
+sub existStr
+{
+    my $hash = shift;
+    my $k1 = shift;
+    my $k2 = shift;
+
+#    print "\thash = ", $$hash{$k1}, "\n";
+#    print "\t\t $k1 exists\n" if (exists $$hash{$k1});
+
+    if (exists $$hash{$k1} and index($$hash{$k1}, "\000$k2\000") >= 0)
+    {
+	return 1;
+    }
+    else
+    {
+	return 0;
+    }
+}
+
+
+############################################################
+# creates sparse file with specified size
+# deletes existing file
+# returns 1 if success / returns 0 if no success
+sub createSparseFile
+{
+    my $file = shift;
+    my $size = shift;
+    my $prLog = shift;
+
+    if (-e $file)
+    {
+	return 0 if unlink $file != 1;
+    }
+
+    unless (sysopen(TO, $file, O_CREAT | O_WRONLY))
+    {
+	$prLog->print('-kind' => 'E',
+		      '-str' => ["cannot create sparse file <$file>"]);
+	return 0;
+    }
+
+    sysseek(TO, $size - 1, 0);
+
+    my $buffer = pack('C', 0);
+
+    unless (syswrite(TO, $buffer))
+    {
+	$prLog->print('-kind' => 'E',
+		      '-str' =>
+		      ["cannot syswrite when creating sparse file <$file>"]);
+	return 0;
+    }
+
+    close(TO);
+    return 1;
+}
+
+
+############################################################
+# read pod2text
+sub getPod2Text
+{
+    my $file = shift;
+    my $prLog = shift;    # if not set, forkProc uses STDERR
+
+    my $p2t = forkProc->new('-exec' => 'pod2text',
+			    '-param' => [$file],
+			    '-outRandom' => "/tmp/pod2text-",
+			    '-prLog' => $prLog);
+    $p2t->wait();
+    my $out = $p2t->getSTDERR();
+    if (@$out > 0)
+    {
+	if ($prLog)
+	{
+	    $prLog->print('-kind' => 'E',
+			  '-str' => ["STDERR of <pod2text>:", @$out])
+		if (@$out > 0);
+	}
+	else
+	{
+	    print STDERR "STDERR of <pod2text>:",
+	    join("\n", @$out), "\n";
+	}
+	exit 1;
+    }
+
+    $out = $p2t->getSTDOUT();
+
+    return join("\n", grep(!/^\s*$/, @$out)) . "\n";
+}
+
+
 ############################################################
 # Objekt kann zum (wiederholten) Abfragen von Informationen
 # ber eine Datei verwendet werden.
@@ -449,7 +801,8 @@ sub new
     my ($self) = {};
 
     my (%params) = ('-filename'      => undef,
-		    '-prLog'         => undef
+		    '-prLog'         => undef,
+		    '-tmpdir'        => '/tmp'
 		    );
 
     &::checkObjectParams(\%params, \@_, 'singleFileInfo::new',
@@ -482,6 +835,7 @@ sub getInfo
 			 ['-kind']);
     my $kind = $params{'-kind'};
     my $prLog = $self->{'prLog'};
+    my $tmpdir = $self->{'tmpdir'};
 
     if ($kind eq 'md5')
     {
@@ -491,8 +845,8 @@ sub getInfo
 	}
 
 	my $f = forkProc->new('-exec' => 'md5sum',
-			      '-stdout' => "/tmp/out.$$",
-			      '-stderr' => "/tmp/err.$$",
+			      '-stdout' => "$tmpdir/out.$$",
+			      '-stderr' => "$tmpdir/err.$$",
 			      '-param' => [$self->{'filename'}],
 			      '-prLog' => $prLog);
 	$f->wait();
@@ -554,6 +908,8 @@ sub new
 		    '-followLinks'         => 0, # nicht folgen, wenn 1, dann
 		                                 # in erster Ebene folgen, mehr
 		                                 # geht bisher nicht
+		    '-stayInFileSystem'    => 0, # don't leave file system, but
+		                                 # consider --followLinks
 		    '-prLog'               => undef,
 		    '-prLogError'          => 'E',
 		    '-prLogWarn'           => 'W',
@@ -599,6 +955,19 @@ sub new
     $self->{'depths'} = \@depths;
     $self->{'printedDepth'} = -1;
 
+    my %stayInFileSystemHash;
+    $self->{'stayInFileSystemHash'} = \%stayInFileSystemHash;
+    if ($self->{'stayInFileSystem'})
+    {
+	my $dirs = $self->{'dirs'};
+	foreach my $d (@$dirs)
+	{
+	    my $device = (stat($d))[0];    # device number of filesystem
+	    $stayInFileSystemHash{$device} = $d;
+#print "(1) $device --> $d\n";
+	}
+    }
+
     bless $self, $class;
 }
 
@@ -646,6 +1015,8 @@ sub readDir
     my $except = $self->{'except'};
     my $include = $self->{'include'};
     my $includeDirs = $self->{'includeDirs'};
+    my $stayInFileSystem = $self->{'stayInFileSystem'};
+    my $stayInFileSystemHash = $self->{'stayInFileSystemHash'};
     my $ignoreReadError = $self->{'ignoreReadError'};
 
 
@@ -717,15 +1088,7 @@ sub readDir
     local *DIR;
     unless (opendir(DIR, $dir))
     {
-	if ($ignoreReadError eq 'no')
-	{
-	    $prLog->print('-kind' => $prLogErr,
-			  '-str' =>
-			  ["cannot opendir <$dir>, exiting",
-			   "check your source and clean your backup!"],
-			  '-exit' => $exit);
-	}
-	elsif ($ignoreReadError eq 'onlyPrintMessage')
+	if ($ignoreReadError)
 	{
 	    $prLog->print('-kind' => $prLogErr,
 			  '-str' => ["cannot opendir <$dir>"]);
@@ -733,30 +1096,54 @@ sub readDir
 	}
 	else
 	{
+	    $prLog->print('-kind' => $prLogErr,
+			  '-str' =>
+			  ["cannot opendir <$dir>, " .
+			   "you may want to set option ignoreReadError to " .
+			   "make this message a warning"]);
+#			  '-exit' => $exit);
 	    return;
 	}
     }
 
     my $entry;
     my @notPlainFiles;
+#########!!!!!!!!
+##my $output = undef;
+##print "reading $dir\n";
     while ($entry = readdir DIR)
     {
 	next if ($entry eq '.' or $entry eq '..');
 	$entry = $dir . '/' . $entry;
+#########!!!!!!!!
+##print "$output\n" if $output;
+##$output = $entry;
+
+#	my $mode = (stat($entry))[2];
+#	if (::S_ISLNK($mode))
 	if (-l $entry)
 	{
 	    if ($self->{'followLinks'} > $depth and
                -d "$entry/.")
 	    {
+#########!!!!!!!!
+##$output = "dir_a $output";
 		if (exists $$except{&::absolutePath($entry)})
 		{
 		    $prLog->print('-kind' => 'I',
-				  '-str' => ["ignoring directory <$entry>"])
+				  '-str' => ["ignoring directory <$entry>, " .
+				  "because of option exceptDir"])
 			if $self->{'verbose'};
 		    push @$files, $entry;
 		    push @$types, 'd';
 		    next;
 		}
+		if ($stayInFileSystem)
+		{
+		    my $device = (stat($entry))[0];
+		    $$stayInFileSystemHash{$device} = $entry;
+#print "(2) $device --> $entry\n";
+		}
 		push @$files, $entry;
 		push @$types, 'd';
 		push @$dirs, $entry;
@@ -764,19 +1151,55 @@ sub readDir
 	    }
 	    else
 	    {
+#########!!!!!!!!
+##$output = "symlink $output";
 		push @$files, $entry;
 		push @$types, 'l';
 	    }
 	    next;
 	}
 	unless (-r $entry)
+#	unless ($mode & ::S_IREAD and $> != 0)  # $> -> effective user id
 	{
-	    $prLog->print('-kind' => $prLogWarn,
-			  '-str' => ["no permissions to read <$entry>"]);
+	    next
+		if exists $$except{&::absolutePath($entry)};
+
+	    if ($ignoreReadError)
+	    {
+		$prLog->print('-kind' => $prLogWarn,
+			      '-str' => ["no permissions to read <$entry>"]);
+	    }
+	    else
+	    {
+		$prLog->print('-kind' => $prLogErr,
+			      '-str' => ["no permissions to read <$entry>, " .
+			      "you may want to set option ignoreReadError to " .
+			      "make this message a warning"]);
+#			      '-exit' => 1);
+	    }
 	    next;
 	}
 	if (-d $entry)       # Dieses Directory mu beim Kopieren
+#	if (::S_ISDIR($mode))  # Dieses Directory mu beim Kopieren
 	{                      # z.B. mu angelegt werden!
+#########!!!!!!!!
+##$output = "dir $output";
+	    if ($stayInFileSystem)
+	    {
+		my $device = (stat($entry))[0];
+		if (not exists $$stayInFileSystemHash{$device})
+		{
+		    $prLog->print('-kind' => 'I',
+				  '-str' => ["ignoring directory <$entry>, " .
+				  "because of option stayInFileSystem"])
+			if $self->{'verbose'};
+		    push @$files, $entry;
+		    push @$types, 'd';
+#print "(3) nix $device --> $entry\n";
+		    next;
+		}
+#print "(2) $device --> $entry\n";
+	    }
 	    push @$files, $entry;
 	    push @$types, 'd';
 	    next if exists $$except{&::absolutePath($entry)};
@@ -785,31 +1208,46 @@ sub readDir
 	    next;
 	}
 	if (-f $entry)
+#	if (::S_ISREG($mode))
 	{
+#########!!!!!!!!
+##$output = "file $output";
 	    push @$files, $entry;
 	    push @$types, 'f';
 	    next;
 	}
 	if (-p $entry)      # named pipe
+#	if (::S_ISFIFO($mode))      # named pipe
 	{
+#########!!!!!!!!
+##$output = "pipe $output";
 	    push @$files, $entry;
 	    push @$types, 'p';
 	    next;
 	}
 	if (-S $entry)      # socket
+#	if (::S_ISSOCK($mode))      # socket
 	{
+#########!!!!!!!!
+##$output = "socket $output";
 	    push @$files, $entry;
 	    push @$types, 'S';
 	    next;
 	}
 	if (-b $entry)      # block special file
+#	if (::S_ISBLK($mode))      # block special file
 	{
+#########!!!!!!!!
+##$output = "blockSpecial $output";
 	    push @$files, $entry;
 	    push @$types, 'b';
 	    next;
 	}
 	if (-c $entry)      # character special file
+#	if (::S_ISCHR($mode))      # character special file
 	{
+#########!!!!!!!!
+##$output = "charSpecial $output";
 	    push @$files, $entry;
 	    push @$types, 'c';
 	    next;
@@ -818,6 +1256,9 @@ sub readDir
 		      '-str' => ["unsupported file type for <$entry>"]);
     }
 
+#########!!!!!!!!
+##print "$output\n" if $output;
+
     closedir DIR;
 }
 
