@@ -1,7 +1,7 @@
 # -*- perl -*-
 
 #
-#   Copyright (C) Heinz-Josef Claes (2001-2012)
+#   Copyright (C) Heinz-Josef Claes (2001-2022)
 #                 hjclaes@web.de
 #
 #   This program is free software: you can redistribute it and/or modify
@@ -20,8 +20,6 @@
 
 
 
-push @VERSION, '$Id: forkProc.pl 362 2012-01-28 22:11:13Z hjc $ ';
-
 use Carp;
 use strict;
 
@@ -66,17 +64,18 @@ sub waitForFile
 
     my @intervall = (.01, .01, .02, .02, .04, .05, .05,
 		      .05, .05, .05, .05, .05, .05, .05, .05,
-		     .05, .05, .05, .05, .05, .05, .05, .05); # 1 second
+		     .05, .05, .05, .05, .05, .05, .05, .05, 0); # 1 second
 
-    if (not -e $existingFile)
+    if ((not -e $existingFile) or (stat($existingFile))[7] == 0)
     {
 	my $i;
 	foreach $i (@intervall)
 	{
 	    select(undef, undef, undef, $i);
-	    if (-e $existingFile)
+	    if (-e $existingFile and (stat($existingFile))[7] > 0)
 	    {
-		$i = 0;
+		select(undef, undef, undef, 2 * $intervall[0]);
+#		system "/bin/sync";
 		last;
 	    }
 	}
@@ -84,7 +83,7 @@ sub waitForFile
 	if ($i == 0)
 	{
 	    system "/bin/sync";
-	    return not -e $existingFile;
+	    return ((not -e $existingFile) or (stat($existingFile))[7] == 0);
 	}
     }
 
@@ -92,6 +91,46 @@ sub waitForFile
 }
 
 
+############################################################
+sub detachFromTerminal
+{
+    my $prLog = shift;
+    my $verbose = shift;
+    my $pidfile = shift;
+
+    return if getppid() == 1;     # ist schon ein daemon
+
+    my $i = fork();
+    exit 0 if ($i == 0);           # child process beenden
+
+    unless (defined $i or POSIX::setsid())
+    {
+	unlink $pidfile if defined $pidfile;
+	$prLog->print('-kind' => 'E',
+		      '-str' => ["cannot daemonize, exiting"],
+		      '-exit' => 1);
+    }
+
+    my $i = fork();
+    exit 0 if ($i > 0);           # parent process beenden
+
+    ## Change working directory
+    chdir "/";
+
+    ## Clear file creation mask
+    umask 0;
+
+    ## Reopen stderr, stdout, stdin to /dev/null
+    open(STDIN,  "+>/dev/null");
+    open(STDOUT, "+>&STDIN");
+    open(STDERR, "+>&STDIN");
+
+    $prLog->print('-kind' => 'I',
+		  '-str' => ["daemonizing"])
+	if $verbose;
+}
+
+
 ############################################################
 package simpleFork;
 
@@ -233,13 +272,17 @@ sub new
     my $err = $params{'-prLogError'};
     my $ex = $params{'-exitIfError'};
 
+    die "use one options of <-outRandom> or <-stdout / -stderr> at" .
+	"__FILE__, __LINE__"
+	unless defined $outRandom or ($stdout and $stderr);
+
     # zufllig Dateinamen erstellen, falls ntig
-    if ($outRandom ne undef)
+    if (defined $outRandom)
     {
 	$self->{'stdout'} = $stdout = &::uniqFileName($outRandom)
-	    if $stdout eq undef;
+	    unless defined $stdout;
 	$self->{'stderr'} = $stderr = &::uniqFileName($outRandom)
-	    if $stderr eq undef;
+	    unless defined $stderr;
     }
 
     my $child;
@@ -351,6 +394,7 @@ sub wait
 	$self->{'fd'} = undef;
 
 	waitpid $self->{'pid'}, 0;
+#	waitpid $self->{'pid'}, WNOHANG;
 	$self->{'status'} = $? >> 8 if $self->{'status'} eq undef;
     }
 }
@@ -653,7 +697,7 @@ sub new
 		    '-delStdout'   => 'yes', # lsche Datei fr stdout
 		                             # automatisch
 
-		    '-prLog'       => undef,
+		    '-prLog'       => undef, # if not set, uses STDERR
 
 		    '-info'        => undef, # beliebige zustzliche Info
 
@@ -662,7 +706,7 @@ sub new
 		    );
 
     &::checkObjectParams(\%params, \@_, 'forkProc::new',
-			 ['-exec', '-prLog']);
+			 ['-exec']);
     &::setParamsDirect($self, \%params);
 
     my $proc = $params{'-exec'};
@@ -677,12 +721,12 @@ sub new
     my $ex = $params{'-exitIfError'};
 
     # zufllig Dateinamen erstellen, falls ntig
-    if ($outRandom ne undef)
+    if (defined $outRandom)
     {
 	$self->{'stdout'} = $stdout = &::uniqFileName($outRandom)
-	    if $stdout eq undef;
+	    unless defined $stdout;
 	$self->{'stderr'} = $stderr = &::uniqFileName($outRandom)
-	    if $stderr eq undef;
+	    unless defined $stderr;
     }
 
     # jetzt forken
@@ -693,9 +737,16 @@ sub new
 	{
 	    unless (chdir "$dir")
 	    {
-		$prLog->print('-kind' => $err,
-			      '-str' =>
-			      ["Cannot chdir to <$dir> for <$proc>"]);
+		if ($prLog)
+		{
+		    $prLog->print('-kind' => $err,
+				  '-str' =>
+				  ["Cannot chdir to <$dir> for <$proc>"]);
+		}
+		else
+		{
+		    print STDERR "Cannot chdir to <$dir> for <$proc>\n";
+		}
 		exit $ex;
 	    }
 	}
@@ -703,9 +754,16 @@ sub new
 	{
 	    unless (sysopen(STDIN, $stdin, 0))
 	    {
-		$prLog->print('-kind' => $err,
-			      '-str' =>
-			      ["Cannot read <$stdin> when starting <$proc>"]);
+		if ($prLog)
+		{
+		    $prLog->print('-kind' => $err,
+				  '-str' =>
+				  ["Cannot read <$stdin> when starting <$proc>"]);
+		}
+		else
+		{
+		    print STDERR "Cannot read <$stdin> when starting <$proc>\n";
+		}
 		exit $ex;
 	    }
 	}
@@ -713,9 +771,16 @@ sub new
 	{
 	    unless (open STDOUT, "> $stdout")
 	    {
-		$prLog->print('-kind' => $err,
-			      '-str' =>
-			      ["Cannot write <$stdout> when starting <$proc>"]);
+		if ($prLog)
+		{
+		    $prLog->print('-kind' => $err,
+				  '-str' =>
+				  ["Cannot write <$stdout> when starting <$proc>"]);
+		}
+		else
+		{
+		    print STDERR "Cannot write <$stdout> when starting <$proc>\n";
+		}
 		exit $ex;
 	    }
 	    chown 0600, $stdout;
@@ -724,19 +789,32 @@ sub new
 	{
 	    unless (open STDERR, "> $stderr")
 	    {
-		$prLog->print('-kind' => $err,
-			      '-str' =>
-			      ["Cannot dup stderr when starting $proc"]);
+		if ($prLog)
+		{
+		    $prLog->print('-kind' => $err,
+				  '-str' =>
+				  ["Cannot dup stderr when starting $proc"]);
+		}
+		else
+		{
+		    print STDERR "Cannot dup stderr when starting $proc\n";
+		}
 		exit $ex;
 	    }
 	    chown 0600, $stderr;
 	}
 
-	$proc =~ s/(\s)/\\$1/g;
 	unless (exec $proc, @$par)
 	{
-	    $prLog->print('-kind' => $err,
-			  '-str' => ["cannot exec $proc @$par"]);
+	    if ($prLog)
+	    {
+		$prLog->print('-kind' => $err,
+			      '-str' => ["cannot exec $proc @$par"]);
+	    }
+	    else
+	    {
+		print STDERR "cannot exec $proc @$par\n";
+	    }
 	    exit $ex;
 	}
     }
@@ -851,10 +929,18 @@ sub signal
 
     my $ret = kill $params{'-value'}, $self->{'pid'};
 
-    if ($ret) {
-	$self->{'prLog'}->print(-kind => 'W',
-				-str =>
-				["signaling process " . $self->{'pid'}]);
+    if ($ret)
+    {
+	if ($self->{'prLog'})
+	{
+	    $self->{'prLog'}->print(-kind => 'W',
+				    -str =>
+				    ["signaling process " . $self->{'pid'}]);
+	}
+	else
+	{
+	    print STDERR "signaling process " . $self->{'pid'} . "\n";
+	}
 	$self->delSTDOUT();
     }
     return $ret;
@@ -1049,8 +1135,7 @@ sub add_block
     my $sched =
 	tinyWaitScheduler->new('-firstFast'     => $self->{'firstFast'},
 			       '-noOfWaitSteps' => $self->{'noOfWaitSteps'},
-			       '-prLog'         => $self->{'prLog'},
-			       '-noOfWaitSteps' => 10);
+			       '-prLog'         => $self->{'prLog'});
 
     my $old;
     $sched->wait() until
@@ -1189,10 +1274,10 @@ sub new
     my $self = {};
 
     my (%params) = ('-maxWaitTime'   => 1,     # in seconds (-> add())
-		    '-noOfWaitSteps' => 10,
+		    '-noOfWaitSteps' => 100,
 		    '-firstFast'     => 1,     # falls 1, beim ersten Mal nicht warten
                     '-debug'         => 0,
-                    '-prLog'         => undef,
+                    '-prLog'         => undef
                    );
 
     &::checkObjectParams(\%params, \@_, 'tinyWaitScheduler::new',
